/*
 *
 * Date:        19/04/2015
 * Info:        Log cleaner 
 * Contact:     mailto: <luca.e [at] seeweb.it>
 * Author:      Luca Ercoli
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

#include <utmp.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>
#include <errno.h>
#include <time.h>


void err_sys(const char *, ...);
void wipe_utmp(char *username, char *tty);
void wipe_wtmp(char *username, char *tty, char *data);
void usage();
int main(int argc, char *argv[]);


void err_sys(const char *fmt, ...)
{
	va_list ap;

	va_start(ap, fmt);
	vfprintf(stderr, fmt, ap);
	fprintf(stderr, ": %s\n", strerror(errno));
	va_end(ap);

	exit(EXIT_FAILURE);
}


void wipe_utmp(char *username, char *tty)
{
	int 		fd1;
	struct utmp	ut;

	if ( (fd1 = open(_PATH_UTMP, O_RDWR)) < 0 ) {
		err_sys("ERROR: Opening %s\n", _PATH_UTMP);
	}

	while ( read(fd1, &ut, sizeof(ut)) > 0) {
		if ( !strncmp(ut.ut_name, username, strlen(username)) ){

			if (!tty || (tty && !strncmp(ut.ut_line, tty, strlen(tty)) ) ) {

				bzero((char *) &ut, sizeof(ut));

				if (lseek(fd1, (int) -sizeof(ut), SEEK_CUR) == EXIT_FAILURE){
					close(fd1);
					err_sys("lseek error");
				}

				if (write(fd1, &ut, sizeof(ut)) == EXIT_FAILURE){
					close(fd1);
					err_sys("write error");
				}
			}
		}
	}

	printf("UTMP: Cleaned!\n");
}


void wipe_wtmp(char *username, char *tty, char *data)
{
	int             fd1;
	struct utmp     ut;

	if ( (fd1 = open(WTMP_FILE, O_RDWR)) < 0 ) {
		fprintf(stderr, "ERROR: Opening %s\n", WTMP_FILE);
		return;
	}

	time_t curtime;
	struct tm *loctime;

	char *mesi[]={"Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"};
	char *giorni[]={"Sun", "Mon","Tue","Wed","Thu","Fri","Sat"};
	char buffer[100];

	while ( (read (fd1, &ut, sizeof(ut))) > 0) {

		curtime=ut.ut_tv.tv_sec;
		loctime = localtime (&curtime);

		memset(buffer,'\0', sizeof(buffer));

		snprintf (buffer,sizeof(buffer),"%s %s %2d %02d:%02d",giorni[loctime->tm_wday],mesi[loctime->tm_mon],loctime->tm_mday,loctime->tm_hour,loctime->tm_min);

		if (!strncmp(ut.ut_name, username, strlen(username)))
			if (!tty || (tty && !strncmp(ut.ut_line, tty, strlen(tty)) && !strncmp(data,buffer,strlen(buffer))) ) {


				printf("WTMP: Cleaned!\n");
				bzero((char *) &ut, sizeof(ut));
				lseek(fd1, (long) -(sizeof(ut)), SEEK_CUR);
				write(fd1, &ut, sizeof(ut));

				break;
			}
	}

	lseek(fd1, (long) -(sizeof(ut) * 2), SEEK_CUR);
	close(fd1);
}


void usage(char *prog)
{
	
	printf("USAGE: %s [u|w] args\n",prog);
	printf("\n");
	printf("Modifica UTMP:\n");
	printf("\tElimina tutte le entry:    %s u [username]\n",prog);
	printf("\tElimina la singola voce:   %s u [username] [tty]\n",prog);
	printf("\n");
	printf("Modifica WTMP:\n");
	printf("\tCancella dal log una singola entry:   %s w [username] [tty] \"[data]\"\n",prog);
	printf("\n");

	exit(EXIT_SUCCESS);
}


int main(int argc, char *argv[])
{

	printf ("\n\n*** Seeweb: server, hosting e servizi cloud (http://www.seeweb.it/) ***\n\n");

	char	c;

	if (argc < 3)
		usage(argv[0]);
	c = toupper(argv[1][0]);

	switch (c) {

		case 'U' :

			if (strlen(argv[2]) > UT_NAMESIZE){
				err_sys("username too large");
			}

			if (argc == 3){
				wipe_utmp(argv[2], (char *) NULL);
			}
			if (argc == 4){

				if (strlen(argv[3]) > UT_LINESIZE){
					err_sys("tty name too large");	
				}
				wipe_utmp(argv[2], argv[3]);
			}

			break;

		case 'W' : 
			if (argc < 5) usage(argv[0]);
			wipe_wtmp(argv[2], argv[3], argv[4]);
			break;
	}

	return(EXIT_SUCCESS);
}
